__version__ = '0.52'

import unittest
import logging
import StringIO

import cssutils.lexer 


class CSSLexerTestCase(unittest.TestCase):

    def setUp(self):
        # logger for lexer
        log = logging.getLogger('lexertest')
        self.slog = StringIO.StringIO()
        hdlr = logging.StreamHandler(self.slog)
        formatter = logging.Formatter('%(levelname)s\t%(message)s')
        hdlr.setFormatter(formatter)
        log.addHandler(hdlr)
        log.setLevel(logging.DEBUG)
        # init lexer
        self.lex = cssutils.lexer.StyleSheetLexer(log)

    def test_normalizeS(self):
        # leave comments, normalize only rules
        com1 = "/*  \n  \f  \r  \t  a  comment  with  whitespace  */"
        com2 = "\t\f\r\n   /*  \n  \f  \r  \t  a  comment  with  whitespace  */  \t\f\r\n  "
        # nested comment
        com3 = "/* /* \n  \f  \r  \t  a  comment  with  whitespace  */"
        self.assertEqual(com1, self.lex._normalizeS(com1))
        self.assertEqual(com1, self.lex._normalizeS(com2))
        self.assertEqual(com1, self.lex._normalizeS(com3))

        # rule
        css = norm = "x, * { color: red; } * /"
        self.assertEqual(css, self.lex._normalizeS(css))

        css1 = "x, *  {  color:  red; \f  \n  \r  \t  }  *  /  "
        self.assertEqual(norm, self.lex._normalizeS(css1))

        css2 = "x, *  {  color:  red; \f  \n  \r  \t  } *   /"
        self.assertEqual(norm, self.lex._normalizeS(css2))

        # mixed
        m1 = com1 + css1 + com2 + css2     
        exp = com1 + norm + com1 + norm
        self.assertEqual(exp, self.lex._normalizeS(m1))

        # check log
        self.assertEqual('ERROR\tNested', self.slog.getvalue()[:12])        

    def test_basecss(self):
        cssText = """
            <!--
            /* comment */
            -->
            <!--
            a { color: red;}
            -->
            """
        expectedtokens = [
            (u'Comment', 'comment'),
            (u'StyleRule', 'a'),
            (u'Property', 'color'),
            (u'Value', 'red'),
            (u'BLOCKEND', '}')]
        actualtokens = self.lex.lex(cssText)
        #log = open(self.logfile).read()
        #self.assertEqual(expectedtokens, actualtokens)
        #self.assertEqual('', log)

    def test_media(self):
        cssText = """
            @media all, print {
            /* comment */
            a { color: red;}
            }
            """
        expectedtokens = [(u'@media', 'all, print'),
            (u'StyleRule', '/* comment */a'),
            (u'Property', 'color'),
            (u'Value', 'red'),
            (u'BLOCKEND', '}'),
            (u'@media-END', '}')]
        actualtokens = self.lex.lex(cssText)
        self.assertEqual(expectedtokens, actualtokens)
        self.assertEqual('', self.slog.getvalue())        

        actualtokens = self.lex.lex(cssText)
        self.assertEqual(expectedtokens, actualtokens)
        self.assertEqual('', self.slog.getvalue())        

    def test_unknownSimple(self):
        cssText = """@namespace url(test.x);"""
        expectedtokens = [(u'UNKNOWN', '@namespace url(test.x);')]
        actualtokens = self.lex.lex(cssText)
        self.assertEqual(expectedtokens, actualtokens)
        self.assertEqual('', self.slog.getvalue())        

    def test_unknownWithBlock(self):
        cssText = """@whatever x { a block: with something; }"""
        expectedtokens = [(u'UNKNOWN', '@whatever x { a block: with something; }')]
        actualtokens = self.lex.lex(cssText)
        self.assertEqual(expectedtokens, actualtokens)
        self.assertEqual('', self.slog.getvalue())        

    def test_unknownError(self):
        cssText = """@whatever x ing }"""
        expectedtokens = []
        actualtokens = self.lex.lex(cssText)
        self.assertEqual(expectedtokens, actualtokens)
        self.assertEqual(True, self.slog.getvalue().startswith(
            'ERROR\t(Lexer)End of block but'))        


if __name__ == '__main__':
    unittest.main() 
